# Copyright (C) 2007 Creative Commons
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import ccwidget
import os
import re
import gtk
import pango
import collections
import html2text
import logging
import StringIO
from gettext import gettext as _
import sugar.activity.activity
from sugar.graphics.toolbutton import ToolButton

FONT='Sans Serif 12'

CONFDIR=os.path.join(os.environ['SUGAR_ACTIVITY_ROOT'], 'conf')
BLOCKER_FILE='blocker-file'
CAN_FILE='can'

SERVICE='org.creativecommons.LicensingActivity'
IFACE = SERVICE
PATH = '/org/creativecomons/LicensingActivity'

_VIEW_SOURCE_KEY_CODE = 0x1008FF1A
_TOOLBAR_BROWSE = 1

BLOCKER_TEXT_FILE='blocker.txt'
PANELS_DIR='panels'

_logger = logging.getLogger('licensing-activity')

import sys
class FakeWriter:
    def write(self, thing):
        _logger.warning(thing)

sys.stdout = FakeWriter()

number_format = '%d'
def panel_png(n):
    return os.path.join(PANELS_DIR, (number_format % n) + '.png')
def panel_txt(n):
    return os.path.join(PANELS_DIR, (number_format % n) + '.txt')

def is_number(s):
    return bool(re.match('\d+', s))

class LicensingActivity(sugar.activity.activity.Activity):

    def read_button_callback(self, whatever):
        canvas = gtk.VBox()
        text = _(open(BLOCKER_TEXT_FILE).read()).strip().replace('\n\n\n', '\n\n')
        textbuf = gtk.TextBuffer()
        textbuf.set_text(text)

        textview = gtk.TextView(textbuf)
        textview.set_wrap_mode(gtk.WRAP_WORD)
        textview.set_editable(False)
        textview.set_size_request(width=-1, height=700)
        textview.modify_font(pango.FontDescription(FONT))
        textview.show()

        canvas.pack_start(textview)

        ok_but = gtk.Button(stock=gtk.STOCK_OK)
        ok_but.connect("clicked", self.pre_load_blocker_button_click, True)
        subcanvas = gtk.HBox()
        ok_but.show()
        subcanvas.pack_start(ok_but)
        subcanvas.show()

        canvas.pack_start(subcanvas)

        canvas.show()
        self.set_canvas(canvas)

    def license_button_callback(self, whatever):
        return self.load_license_chooser()

    def comic_button_callback(self, whatever):
        self.load_comic_canvas()

    def _update_activity_toolbar(self, activity_toolbar):
        if self._home_button is None:
            self._home_button = ToolButton('view-refresh') # This makes a little sense.
            self._home_button.set_tooltip(_("Go to main menu"))
            self._home_button.set_sensitive(True)
            self._home_button.connect('clicked', self.main_menu_callback)
            activity_toolbar.insert(self._home_button, 1)
            self._home_button.show()

    def add_can(self, things_we_can_now):
        if not things_we_can_now:
            print "Discarding false add_can call value: %s", repr(things_we_can_now)
            return
        if str(things_we_can_now) == things_we_can_now:
            things_we_can_now = [things_we_can_now]
        can_needs_update = False
        for can_do in things_we_can_now:
            if can_do not in self._can:
                can_needs_update = True
        if can_needs_update:
            self._can.update(things_we_can_now)
            self._save_can()
        print "Successefully add_can'd %s", repr(things_we_can_now)

    def _save_can(self):
        can_string = '\n'.join(sorted(list(self._can)))
        fd = self._get_conf_file(CAN_FILE, 'w')
        fd.write(can_string)
        fd.close()

    def _get_conf_file(self, filename, mode='r'):
        if not os.path.exists(CONFDIR):
            _logger.warning("Huh, had to make CONFDIR.")
            os.makedirs(name=CONFDIR, mode=0755)
        full_filename = os.path.join(CONFDIR, filename)
        if mode.startswith('r'):
            if not os.path.exists(full_filename):
                return StringIO.StringIO() # dummy empty file
        return open(os.path.join(CONFDIR, filename), mode)

    def _load_can(self):
        self._can.update(self._get_conf_file(CAN_FILE).read().split('\n'))
        self.add_can('read')

    def handle_can(self):
        string2buttons = collections.defaultdict(list)
        string2buttons.update( {
            'read': [self._main_menu_read_button],
            'comic': [self._main_menu_comic_button],
            'license': [self._main_menu_license_button]})
        for can_do in self._can:
            for button in string2buttons[can_do]:
                button.set_sensitive(True)
 
    def finished_read_portion(self):
        self.add_can('comic')

    def finished_comic_portion(self):
        self.add_can('license')

    def __init__(self, handle):
        # Activity Menu buttons
        self._home_button = None

        # Main menu buttons
        self._main_menu_read_button = None
        self._main_menu_comic_button = None
        self._main_menu_license_button = None

        self._can = set() # a set of things that can be done
        self._load_can()

        sugar.activity.activity.Activity.__init__(self, handle)
        
        _logger.warning('Starting the CC activity')

        toolbox = sugar.activity.activity.ActivityToolbox(self)
        activity_toolbar = toolbox.get_activity_toolbar()
        self._update_activity_toolbar(activity_toolbar)

        self.set_toolbox(toolbox)
        toolbox.show()

        # Initialize looking_at
        self.looking_at = 0

        # Start off with the first image.
        self.panel_count = self.count_available_panels()

        # Don't load the canvas until the user has agreed
        # to something.
        self.main_menu()

    def get_blocker_value(self):
        ''' Output: The contents of the blocker file, stripped'''
        return self._get_conf_file(BLOCKER_FILE).read().strip()

    def set_blocker_value(self, value):
        ''' Sets the blocker file to have contents "value".'''
        _logger.warning("Found value stored for blocker %s" % self.get_blocker_value())
        if self.get_blocker_value() != value:
            _logger.warning("Going to set the blocker file to value %s" % value)
            fd = self._get_conf_file(BLOCKER_FILE, 'w')
            print >> fd, value
            fd.close()
        else:
            _logger.warning("Blocker file already had value %s" % value)

    def pre_load_blocker_button_click(self, widget, callback_data):
        if callback_data == True:
            self.add_can('comic')
        self.main_menu()

    def main_menu_callback(self, widget):
        return self.main_menu()

    def main_menu(self):
        # FIXME: fix images in buttons!
        canvas = gtk.HBox()
        self._main_menu_read_button = ccwidget.VerticalImageLabelButton(image_filename='icons/one-disclaimer.svg', 
                                               labeltext=_('Read disclaimer'),
                                                click_callback=self.read_button_callback)

        self._main_menu_comic_button = ccwidget.VerticalImageLabelButton(image_filename='icons/two-dinosaur.svg', labeltext=_('See comic'),
                                                click_callback=self.comic_button_callback)

        self._main_menu_license_button = ccwidget.VerticalImageLabelButton(image_filename='icons/three-license.svg', labeltext=_('Choose a license'),
                                                click_callback=self.license_button_callback)

        buttons = [self._main_menu_read_button, self._main_menu_comic_button, self._main_menu_license_button]
        map(lambda thing: thing.show(), buttons)
        map(lambda thing: canvas.pack_start(thing), buttons)
        canvas.show()

        self.handle_can()

        self.set_canvas(canvas)

    def count_available_panels(self):
        ''' Asserts that panel 01 exists and returns the highest panel
        number.'''
        panels = sorted(os.listdir('panels/'))
        pngs = [p.rsplit('.png', 1)[0] for p in panels if p.endswith('.png')]
        number_pngs = sorted([int(s) for s in pngs if is_number(s)])
        _logger.warning(`number_pngs`)
        assert 1 in number_pngs
        return int(number_pngs[-1])

    def load_license_chooser(self):
        import os
        os.environ['LD_LIBRARY_PATH'] = os.environ.get('LD_LIBRARY_PATH', '') + ':built/lib'
        import sys
        sys.path.append('built/lib/python2.5/site-packages/')
        import gui_gtk
        gui_gtk.icon_location = os.environ['SUGAR_BUNDLE_PATH'] + '/built/share/liblicense/icons/'
        canvas = gui_gtk.LicenseWidget('http://creativecommons.org/licenses/by-nc/3.0/')
        canvas.set_license(None) # By default
        canvas.show()
        self.set_canvas(canvas)
        


    def load_comic_canvas(self, number = 1):
        ''' Create a canvas for the image+text of number number,
        set our state to know that we're looking at that one,
        and then show the sucker.

        Also set the booleans self.can_go_back and
        self.can_go_forward.'''

        if self.looking_at == number:
            return # Do nothing if we're already looking at it

        if number > self.panel_count:
            # You've fallen off the edge of the comic!
            self.add_can('license')
            return self.main_menu()

        self.looking_at = number

        canvas = gtk.VBox() # to contain Image Hbox and the Text

        # Make the image and buttons...
        image_and_buttons_hbox = gtk.HBox()

        # Button 1
        left_but = ToolButton('go-left')
        left_but.connect('clicked', lambda b: self.load_comic_canvas(self.looking_at - 1) )
        left_but.props.sensitive = False
        #left_but.show()
        image_and_buttons_hbox.pack_start(left_but)

        # Image
        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(panel_png(number), 1200, 600)
        image = gtk.Image()
        image.set_from_pixbuf(pixbuf)
        image_and_buttons_hbox.pack_start(image)
        #image.show()
        
        # Button 2
        right_but = ToolButton('go-right')
        right_but.connect('clicked', lambda a: self.load_comic_canvas(self.looking_at + 1) )
        right_but.props.sensitive = False
        #right_but.show()
        image_and_buttons_hbox.pack_end(right_but)
        #image_and_buttons_hbox.show()

        canvas.pack_start(image_and_buttons_hbox)
        image_and_buttons_hbox.show_all()

        # Now add text to the canvas.
        text = _(html2text.html2text(open(panel_txt(number)).read()))

        textbuf = gtk.TextBuffer()
        textbuf.set_text(text)
        textview = gtk.TextView(textbuf)


        def c(color):
            return (color.red, color.green, color.blue)
        def guess():
            import random
            widgets = [right_but, image, canvas]
            states = gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_INSENSITIVE, gtk.STATE_PRELIGHT, gtk.STATE_SELECTED
            section_accessors = [lambda thing: thing.fg, lambda thing: thing.bg, lambda thing: thing.base, lambda thing: thing.mid, lambda thing: thing.dark, lambda thing: thing.light, lambda thing: text]
            widget_index, widget = random.choice(list(enumerate(widgets)))
            state_index, state = random.choice(list(enumerate(states)))
            section_index, section_accessor = random.choice(list(enumerate(section_accessors)))
            print widget_index, state_index, section_index, 'roflcopterz'
            return section_accessor(widget.get_style())[state]

        # Steal the background color of a button
        #right_but_style = right_but.get_style()
        #right_but_base_color = right_but_style.bg[gtk.STATE_NORMAL]
        #for state in gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_INSENSITIVE, gtk.STATE_PRELIGHT, gtk.STATE_SELECTED:
        #    print 'bg @ state' , state, ' =', c(right_but_style.bg[state])
        #    print 'fg @ state' , state, ' =', c(right_but_style.fg[state])
        #    print 'base @ state' , state, ' =', c(right_but_style.base[state])
        #    print 'image base @ state' , state, ' =', c(image.get_style().base[state])
        
        
        # Background to usual Sugar gray
        textview.show()
        textview.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse('#c6c3c6'))

        # Pad our box
        textview.set_border_width(10)
        textview.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#c6c3c6")) 

        # Set the font size and not editability       
        textview.modify_font(pango.FontDescription(FONT))
        textview.set_wrap_mode(gtk.WRAP_WORD)
        textview.set_editable(False)
        #textview.show()
        canvas.pack_start(textview)

        canvas.show_all()
        self.set_canvas(canvas)

        # Can go back?
        if self.looking_at == 1:
            # Then there is no back
            self.can_go_back = False
        else:
            self.can_go_back = True

        # Can go forward?  Always - when you fall off, you're specially handled above.
        self.can_go_forward = True

        # Update buttons
        left_but.props.sensitive = self.can_go_back
        right_but.props.sensitive = self.can_go_forward
